package	jp.co.sds_corp.util.jmx;

import java.util.Date;

import javax.management.AttributeChangeNotification;
import javax.management.MBeanRegistration;
import javax.management.MBeanServer;
import javax.management.NotificationBroadcasterSupport;
import javax.management.ObjectName;

import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public abstract class AbstractMBeanService
	extends NotificationBroadcasterSupport
	implements IMBeanService, MBeanRegistration
{
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	private int state;
	protected MBeanServer server;
	private int id = 0;

	public AbstractMBeanService() {
	}

	public abstract String getName();

	public MBeanServer getServer() {
		return server;
	}

	public int getState() {
		return state;
	}

	public String getStateString() {
		return states[state];
	}

	public void init() throws Exception {
		init( null );
	}

	public void init( IConfiguration config ) throws Exception {
		logger.debug( "Initializing:" + getName() );
		try {   
			initService( config );
		} catch (Exception e) {
			logger.error( "Initialization failed", e);
			throw e;
		} finally {
		}
		logger.info( "Initialized:" + getName() );
	}

	public void start() throws Exception {
		if (getState() != STOPPED) {
			return;
		}

		state = STARTING;
		sendNotification(new AttributeChangeNotification(	
				this, id++, new Date().getTime(), getName()+" starting",
				"State", "java.lang.Integer", Integer.valueOf(STOPPED),
				Integer.valueOf(STARTING)));
		logger.debug( "Starting:" + getName() );
		try {
			startService();
		} catch (Exception e) {
			state = STOPPED;
			sendNotification(new AttributeChangeNotification(this, id++,
					new Date().getTime(), getName()+" stopped",
					"State", "java.lang.Integer", Integer.valueOf(STARTING),
					Integer.valueOf(STOPPED)));
			logger.debug( "Stopped", e);
			throw e;
		} finally {
		}
		state = STARTED;
		sendNotification(new AttributeChangeNotification(this, id++,
				new Date().getTime(), getName()+" started",
				"State", "java.lang.Integer", Integer.valueOf(STARTING),
				Integer.valueOf(STARTED)));
		logger.info( "Started:" + getName() );
	}

	public void stop() {
		if (getState() != STARTED) {
			return;
		}

		state = STOPPING;
		sendNotification(new AttributeChangeNotification(this, id++,
				new Date().getTime(), getName()+" stopping",
				"State", "java.lang.Integer", Integer.valueOf(STARTED),
				Integer.valueOf(STOPPING)));
		logger.debug( "Stopping:" + getName() );

		try {
			stopService();
		} catch (Throwable e) {
			logger.debug( "stop service", e);
		}

		state = STOPPED;
		sendNotification(new AttributeChangeNotification(this, id++,
				new Date().getTime(), getName()+" stopped",
				"State", "java.lang.Integer", Integer.valueOf(STOPPING),
				Integer.valueOf(STOPPED)));
		logger.debug( "Stopped:" + getName() );
	}

	public void destroy() {
		if (getState() != STOPPED) {
			stopService();
		}

		logger.debug( "Destroying:" + getName() );
		try {
		   destroyService();
		} catch (Exception e) {
		   logger.debug( "destroy", e);
		}

		logger.debug( "Destroyed:" + getName() );
	}

	public ObjectName preRegister(MBeanServer server, ObjectName name)
		throws java.lang.Exception
	{
		name = getObjectName(server, name);
		this.server = server;
		return name;
	}

	public void postRegister(java.lang.Boolean registrationDone) {
		if (!registrationDone.booleanValue()) {
			logger.debug( "Registration is not done -> destroy" );
			destroy();
		}
	}

	public void preDeregister() throws java.lang.Exception {
	}

	public void postDeregister() {
		destroy();
	}

	protected ObjectName getObjectName(MBeanServer server, ObjectName name)
		throws javax.management.MalformedObjectNameException
	{
		return name;
	}

	public void initService( IConfiguration config ) throws Exception {
	}

	public void startService() throws Exception {
	}

	public void stopService() {
	}

	public void destroyService() {
	}

	protected int nextSequenceNumber() {
		return id++;
	}
}
